@@ -34,6 +34,8 @@ import logging
 import datetime
 from decimal import Decimal
 
+from sqlalchemy.orm.exc import NoResultFound
+
 import edbob
 from edbob.time import localize
 
@@ -703,49 +705,110 @@ class ChangesProcessor(Object):
         session.delete(cost)
 
     def process_clt_chg(self, row, session):
-        q = session.query(model.Customer)
-        q = q.filter(model.Customer.id == row['F1148'])
-        customer = q.first()
-        if customer:
-            customer.emails = []
-            customer.phones = []
-            customer.groups = []
-        else:
+        customer_id = row[u'F1148'].strip()
+        assert customer_id
+        try:
+            customer = session.query(model.Customer)\
+                .filter(model.Customer.id == customer_id)\
+                .one()
+        except NoResultFound:
             person = model.Person()
             person.first_name = row['F1149']
             person.last_name = row['F1150']
-            customer = model.Customer(id=row['F1148'])
+            customer = model.Customer(id=customer_id)
             customer.people.append(person)
             session.add(customer)
 
         customer.name = row['F1155']
 
-        if row['F1172']:
-            customer.add_phone_number(row['F1172'])
-        if row['F1173']:
-            customer.add_phone_number(row['F1173'])
-        if row['F1573']:
-            customer.add_email_address(row['F1573'])
+        # Phone Number
+        phone_number = row[u'F1172']
+        if phone_number:
+            if customer.phones:
+                if customer.phones[0].type == u'Home':
+                    customer.phones[0].number = phone_number
+                else:
+                    customer.phones.insert(0, model.CustomerPhoneNumber(number=phone_number, type=u'Home'))
+            else:
+                customer.phones.append(model.CustomerPhoneNumber(number=phone_number, type=u'Home'))
+        elif customer.phones and customer.phones[0].type == u'Home':
+            session.delete(customer.phones.pop(0))
 
-        F1154 = row['F1154'].strip()
-        if F1154:
-            group = session.query(model.CustomerGroup)\
-                .filter(model.CustomerGroup.id == F1154)\
-                .first()
-            if not group:
-                group = model.CustomerGroup(id=F1154)
-                session.add(group)
-            customer.groups.append(group)
+        # Fax Number
+        fax_number = row[u'F1173']
+        if fax_number:
+            if customer.phones:
+                if customer.phones[0].type == u'Fax':
+                    assert len(customer.phones) == 1
+                    customer.phones[0].number = fax_number
+                elif len(customer.phones) > 1:
+                    assert len(customer.phones) == 2
+                    assert customer.phones[1].type == u'Fax'
+                    customer.phones[1].number = fax_number
+                else:
+                    customer.phones.append(model.CustomerPhoneNumber(number=fax_number, type=u'Fax'))
+            else:
+                customer.phones.append(model.CustomerPhoneNumber(number=fax_number, type=u'Fax'))
+        elif customer.phones:
+            i = len(customer.phones) - 1
+            if customer.phones[i].type == u'Fax':
+                session.delete(customer.phones.pop(i))
 
-        F1777 = row['F1777'].strip()
-        if F1777:
-            group = session.query(model.CustomerGroup)\
-                .filter(model.CustomerGroup.id == F1777)\
-                .first()
-            if not group:
-                group = model.CustomerGroup(id=F1777)
+        # Email Address
+        email_address = row[u'F1573']
+        if email_address:
+            if customer.emails:
+                assert len(customer.emails) == 1
+                customer.emails[0].address = email_address
+            else:
+                customer.emails.append(model.CustomerEmailAddress(address=email_address))
+        elif customer.emails:
+            assert len(customer.emails) == 1
+            session.delete(customer.emails.pop(0))
+
+        def fetch_group(group_id):
+            try:
+                group = session.query(model.CustomerGroup)\
+                    .filter(model.CustomerGroup.id == group_id)\
+                    .one()
+            except NoResultFound:
+                group = model.CustomerGroup(id=group_id)
                 session.add(group)
-            customer.groups.append(group)
+            return group
+
+        # Groups
+        group_ids = [row[u'F1154'].strip(), row[u'F1777'].strip()]
+        group1, group2 = None, None
+        if group_ids[0]:
+            group1 = fetch_group(group_ids[0])
+            if customer.groups:
+                customer._groups[0].group = group1
+            else:
+                customer.groups.append(group1)
+        if group_ids[1]:
+            group2 = fetch_group(group_ids[1])
+            if group1:
+                if len(customer.groups) > 1:
+                    assert len(customer.groups) == 2
+                    customer._groups[1].group = group2
+                else:
+                    customer.groups.append(group2)
+            else:
+                if not customer.groups:
+                    customer.groups.append(group2)
+                elif len(customer.groups) == 1:
+                    customer._groups[0].group = group2
+                else:
+                    assert len(customer.groups) == 2
+                    customer._groups[0].group = group2
+                    session.delete(customer._groups.pop(1))
+        elif group1:
+            if len(customer.groups) > 1:
+                assert len(customer.groups) == 2
+                session.delete(customer._groups.pop(1))
+        elif customer.groups:
+            for i in range(len(customer.groups)):
+                session.delete(customer._groups.pop(0))
 
         return customer
 
