@@ -40,8 +40,8 @@ import edbob
 
 from rattail import sil
 from rattail.core import Object
+from rattail.filemon import Action
 from rattail.files import locking_copy, temp_path, overwriting_move
-from rattail_locsms.deploy import deploy_to_inbox
 from rattail_locsms.sil import SMSWriter
 from rattail_locsms.exceptions import InvalidTransferFileName
 
@@ -92,23 +92,20 @@ def copy_transfer(csv_path):
     locking_copy(csv_path, target_dir)
 
 
-def process_transfer(csv_path, node=None):
+def process_transfer(csv_path, node_outbox_path, save_path=None):
     """
     This function is designed to be invoked by the file monitor, on the
     receiving end of a product transfer between stores.  It will inspect the
     contents of a store transfer export file, and take steps to ensure that the
     products included will contain a valid (and current) cost record with the
     sending store as the vendor.
-
-    Note that there are many caveats here...  Ask me sometime and I'll go into
-    more detail. ;)
     """
 
     edbob.init_modules(['rattail_locsms.db'])
 
-    log.debug("process_transfer: processing file: {0}".format(csv_path))
+    log.debug(u"processing file: {0}".format(csv_path))
     info = parse_filename(csv_path)
-    log.debug("process_transfer: sending store is '{0}', receiving store is '{1}'".format(
+    log.debug(u"sending store is '{0}', receiving store is '{1}'".format(
             info.sending_store, info.receiving_store))
 
     upcs = set()
@@ -118,7 +115,7 @@ def process_transfer(csv_path, node=None):
         upcs.add(row['F01'])
     csv_file.close()
 
-    log.debug("process_transfer: found {0} UPC(s)".format(len(upcs)))
+    log.debug(u"found {0} UPC(s)".format(len(upcs)))
     if not upcs:
         return
 
@@ -129,10 +126,10 @@ def process_transfer(csv_path, node=None):
 
         product = session.query(locsms.Product).get(upc)
         if not product:
-            log.warning("process_transfer: product not found: {0}".format(upc))
+            log.warning(u"product not found: {0}".format(upc))
             continue
         if not product.cost:
-            log.warning("process_transfer: product exists but has no cost: {0}".format(repr(product)))
+            log.warning(u"product exists but has no cost: {0}".format(repr(product)))
             continue
 
         magic_cost = None
@@ -148,7 +145,7 @@ def process_transfer(csv_path, node=None):
             and magic_cost.F220 == product.cost.F220    # Split item code
             and magic_cost.F1795 == product.cost.F1795  # Split item qty
             ):
-            log.debug("process_transfer: product cost is already correct: {0}".format(repr(magic_cost)))
+            log.debug(u"product cost is already correct: {0}".format(repr(magic_cost)))
             continue
 
         cost = Object()
@@ -160,27 +157,25 @@ def process_transfer(csv_path, node=None):
         cost.split_quantity = product.cost.F1795
 
         if magic_cost:
-            log.debug("process_transfer: product cost will be included in CHANGE batch: {0}".format(repr(product)))
+            log.debug(u"product cost will be included in CHANGE batch: {0}".format(repr(product)))
             change_costs.append(cost)
         else:
-            log.debug("process_transfer: product cost will be included in ADD batch: {0}".format(repr(product)))
+            log.debug(u"product cost will be included in ADD batch: {0}".format(repr(product)))
             add_costs.append(cost)
         
     session.close()
 
     if add_costs:
-        deploy_costs(add_costs, 'ADD', node)
+        deploy_costs(add_costs, u'ADD', node_outbox_path, save_path=save_path)
     if change_costs:
-        deploy_costs(change_costs, 'CHANGE', node)
+        deploy_costs(change_costs, u'CHANGE', node_outbox_path, save_path=save_path)
 
 
-def deploy_costs(costs, H12, node):
+def deploy_costs(costs, H12, node_outbox_path, save_path=None):
     """
-    Deploy a batch of new ``COST_TAB`` records (``costs``) to the SMS instance
-    referenced by ``node``.
+    Deploy a batch of new/updated ``COST_TAB`` records to an SMS node.
     """
-
-    log.debug("deploy_costs: processing {0} batch with {1} cost(s)".format(H12, len(costs)))
+    log.debug(u"processing {0} batch with {1} cost(s)".format(H12, len(costs)))
 
     path = temp_path('.sil')
     writer = SMSWriter(path)
@@ -214,12 +209,11 @@ def deploy_costs(costs, H12, node):
 
     writer.write_rows(rows, len(costs))
     writer.close()
-    deploy_to_inbox(path, node)
+    locking_copy(path, node_outbox_path)
 
-    save_path = edbob.config.get('rattail.sw.locsms', 'transfers.sil_backup_dir')
     if save_path:
         save_path = os.path.join(save_path, 'rattail.{0}.sil'.format(batch_id))
-        log.debug("deploy_costs: saving copy of SIL file at: {0}".format(save_path))
+        log.debug(u"saving copy of SIL file at: {0}".format(save_path))
         overwriting_move(path, save_path)
     else:
         os.remove(path)
